<#
WINDOWS 10 SILENT INSTALLER build 36/SEAGULL [X86/X64]
WINDOWS 7 SP1 INPLACE UPGRADE and PS2.0 COMPATIBILITY, UNC/HTTP PATH TEST, OOBE MANAGEMENT AND HOUSEKEEPING FEATURES BY JON NORTH, DATTO, NOVEMBER 2018
UPDATED FOR REBOOT CONTROL, DIRECT ISO ATTACHMENT AND UPGRADE TO ENTERPRISE (from Enterprise or Professional) BY JON NORTH, DATTO, DECEMBER 2018

OSFMount virtual ODD driver distributed under EULA from PassMark Software Pty Ltd
More information: https://www.osforensics.com/tools/mount-disk-images.html
#>

$ErrorActionPreference = "stop"

#user text
write-host "WINDOWS 10 UPGRADE TOOL"
WRITE-HOST "=================================="

if ($env:usrShowOOBE -match "true") {
    $varShowOOBE="full"
} else {
    $varShowOOBE="none"
}

if ($env:usrReboot -eq "false") {$RebootSwitch= "/noreboot"} else {$RebootSwitch=$null}

if (!(($env:usrProdKey -eq $null) -or ($env:usrProdKey -eq ""))) {$ProdKeySwitch= "/pkey $env:usrProdKey"} else {$ProdKeySwitch=$null}



# Check architecture and get correct version of OSFMount .com and .sys driver files
If (${env:ProgramFiles(x86)}) {
    Rename-Item OSFMount_x64.com OSFMount.com
    Rename-Item OSFMount_x64.sys OSFMount.sys
} else {
    Rename-Item OSFMount_x86.com OSFMount.com
    Rename-Item OSFMount_x86.sys OSFMount.sys
}

#configure service pipe timeout
try {$varServiceTimeout=((Get-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control" -Name ServicesPipeTimeout).ServicesPipeTimeout)}
    catch [System.Exception] {
        write-host "+ Target OS service timeout period configured to 4 minutes."
        New-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control" -Name ServicesPipeTimeout -PropertyType Dword -Value 240000 2>&1>$null
        $varServiceTimeout=((Get-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control" -Name ServicesPipeTimeout).ServicesPipeTimeout)
    }

if ($varServiceTimeout -lt 240000) {
    write-host "+ Target OS service timeout period configured to 4 minutes."
    New-ItemProperty -Path "HKLM:\SYSTEM\CurrentControlSet\Control" -Name ServicesPipeTimeout -PropertyType Dword -Value 240000 -Force 2>&1>$null
}

#check for minimum W7 SP1
$global:varKernel=[System.Diagnostics.FileVersionInfo]::GetVersionInfo("C:\Windows\system32\kernel32.dll").FileBuildPart
if ($varKernel -lt 7601) {
    write-host "`- Error code SGL1:"
    write-host "  This component requires Microsoft Windows 7 SP1 or higher to proceed."
    exit 1
}

write-host "+ Target device OS is Windows 7 SP1 or greater."


#find edition of windows, fail if not the one specified (Professional or Enterprise) unless a ProdKey variable is set
if ($ProdKeySwitch=$null) {
$varEdition=(cscript /nologo C:\windows\system32\slmgr.vbs /dli | select-string -quiet "$env:usrOSType")
if (!$varEdition) {
    write-host "- Error code SGL2:"
    write-host "  The OS detected is not of type $env:usrOSType"
    exit 1
}
write-host "+ Target device OS edition matches $env:usrOSType"
}



#make sure it's licensed (v2)
$varLicence = Get-WmiObject SoftwareLicensingProduct | Where-Object { $_.LicenseStatus -eq 1 } | Select-Object -ExpandProperty Description | select "Windows"
if (!$varLicence) {
    write-host "`- Error code SGL3:"
    write-host "  Windows 10 can only be installed on devices with an active Windows licence."
    exit 1
}

write-host "+ Target device OS is licensed."

#make sure we have enough disk space - installation plus iso hosting
$varSysFree = [Math]::Round((Get-WMIObject -Class Win32_Volume |Where-Object {$_.DriveLetter -eq $env:SystemDrive} | Select -expand FreeSpace) / 1GB)
if ($varSysFree -le 20) {
    write-host "`- Error code SGL4:"
    write-host "  System drive requires at least 20GB: 13 for installation, 7 for the disc image."
    exit 1
}

write-host "+ Target device has at least 20GB of free hard disk space."

#join the parts together to recreate the ISO file, and delete the parts
cmd /c copy /b part*.iso $env:PUBLIC\Win10.iso
Remove-Item part*.iso

#create Housekeeping batch script to delete the ISO, and optionally the Windows.old rollback folder, when Windows 10 first launches
$HousekeepingCMD='@echo off
del "'+$env:PUBLIC+'\Win10.iso"
del "'+$env:PUBLIC+'\Housekeeping.cmd"
'
if ($env:usrCleanup -eq "True") {
    $HousekeepingCMD+=':LoopTilGone
takeown /F \windows.old\$Windows.~BT\* /R /A 
icacls \windows.old\$Windows.~BT\*.* /T /grant everyone:F 
takeown /F \windows.old\$Windows.~WS\* /R /A 
icacls \windows.old\$Windows.~WS\*.* /T /grant everyone:F 
rd \windows.old /q /s || rem
if %ERRORLEVEL% neq 0 goto LoopTilGone
' }
$HousekeepingCMD+='(goto) 2>nul & del "%~f0"
exit
'
$HousekeepingCMD | Out-File $env:PUBLIC\Housekeeping.cmd -Encoding ascii

#find the first unused drive letter
$arrPopDrives="A:`r`n"
$arrPopDrives+="B:`r`n"
foreach ($varDetectLetter in get-wmiobject -class win32_logicaldisk) {$arrPopDrives+=$varDetectLetter.deviceID+"`r`n"}

foreach ($varCandidateLetter in [char[]]([int][char]'E'..[int][char]'Z')) {
    $varCandidateLetter+=":"
    if ($arrPopDrives -notmatch $varCandidateLetter) {
        $varFirstLetter=$varCandidateLetter
        break
    }
}

write-host "`- Mounting image to drive $varFirstLetter`..."

#mount the disc and allow to finish before attempting setup launch
$ArgList="-a -t file -f ""$env:PUBLIC\Win10.iso"" -m "+$varFirstLetter
Start-Process osfmount.com -ArgumentList $ArgList -wait

#install
cmd /c start "" $varFirstLetter`\setup.exe /auto upgrade /quiet /compat IgnoreWarning $ProdKeySwitch $RebootSwitch /PostOOBE "$env:PUBLIC\Housekeeping.cmd" /showOOBE $varShowOOBE

#close
write-host "+ Endpoint has been instructed to install Windows 10."
write-host "`>  For more information on the installation process, please consult the StdErr from this job run."
exit